@@ -1,4 +1,4 @@
 from .pysd import read_vensim, read_xmile, load
 from .py_backend import functions, statefuls, utils, external
-from .py_backend.decorators import subs
+from .py_backend.components import Component
 from ._version import __version__
